import os
import time
import tempfile
from functools import wraps
from filelock import FileLock

class GlobalRateLimiter:
    def __init__(self, interval_sec: float, id: str = 'ks_trade_api_global_rate_limit'):
        """
        interval_sec: 限流间隔（秒）
        lock_name: 跨进程锁文件名
        ts_name: 记录上次调用时间戳的文件名
        """
        tmp_dir = os.path.expandvars(r"%LOCALAPPDATA%\Temp")  # 自动适配平台，例如 Windows 用 C:\Users\xx\AppData\Local\Temp。不能用tempfile.gettempdir()因为多会话会出现/temp/1和/temp不同
        self.interval = interval_sec
        lock_name = f'{id}.lock'
        ts_name = f'{id}.text'
        self.lock_file = os.path.join(tmp_dir, lock_name)
        self.ts_file = os.path.join(tmp_dir, ts_name)
        self.lock = FileLock(self.lock_file)

    def __call__(self, func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            with self.lock:  # 跨进程互斥锁
                now = time.time()
                try:
                    if os.path.exists(self.ts_file):
                        with open(self.ts_file, "r") as f:
                            last = float(f.read().strip())
                    else:
                        last = 0.0
                except Exception:
                    last = 0.0

                wait_time = self.interval - (now - last)
                if wait_time > 0:
                    time.sleep(wait_time)

                try:
                    with open(self.ts_file, "w") as f:
                        f.write(str(time.time()))
                except Exception as e:
                    print(f"[RateLimiter] Failed to write ts_file: {e}")

            return func(*args, **kwargs)
        return wrapper



# __version__ = "0.0.58" # 周期增加月份支持
# __version__ = "0.0.59" # 增加Indicator
# __version__ = "0.0.60" # 增加datatimetype
# __version__ = "0.0.61" # query_hisotry_n由decimal改为返回float
# # __version__ = "0.0.71" # DataTimeType -> Timing
# __version__ = "0.0.63" # 增加很多constant。例如ExecuteStrategy, IndicatorColumn等标准字段
# __version__ = "0.0.64" # 增加INTERVAL2SUBSCRIBE_TYPE。更新ks_utility到版本0.0.32
# __version__ = "0.0.65" # 增加SUBSCRIBE_TYPE2INTERVAL
# __version__ = "0.0.69" # 增加IndicatorData
# __version__ = "0.0.72" # MySubscribeType支持snapshot
# __version__ = "0.0.73" # 合并之前分支
# __version__ = "0.0.74" # get_trading_time默认返回原来交易所时区
# __version__ = "0.1.0" # 全部enum移除中文
# __version__ = "0.1.1" # 增加global_limit_rate跨线程流速控制装饰器
# __version__ = "0.1.2" # 修正global_limit_rate临时目录
# __version__ = "0.1.4" # update version
__version__ = "0.1.5" # 重构globallimit


from abc import ABC, abstractmethod
import os
import pandas as pd
from typing import Union, Tuple, Optional
#SysConfig.set_all_thread_daemon(True)
from ks_utility.logs import LoggerBase
from datetime import datetime
from .constant import *
from ks_trade_api.object import ContractData, MyAccountData, ErrorData, MyPositionData, MyTradeData, MyOrderData
from .constant import (
    Currency as KsCurrency,
    Exchange as KsExchange,
    Direction as KsDirection, 
    OrderType as ksOrderType, 
    Direction as KsDirection,
    Offset as KsOffset, 
    TimeInForce as KsTimeInForce,
    ErrorCode as KsErrorCode,
    Status as KsStatus,
    RET_OK, 
    RET_ERROR, 
    CHINA_TZ,
    
)
from ks_trade_api.utility import extract_vt_symbol
from ks_utility.numbers import to_decimal
import sys
from decimal import Decimal
import uuid
from ks_utility.datetimes import get_ts_int, get_dt_int
from ks_trade_api.object import ErrorData, ErrorCode
import traceback
from threading import Timer
import queue
import itertools
import math

RATES_INTERVAL = 30 # 30秒内30次请求，所以一旦超频，就睡眠30秒
TRIGGER_NUMS = 15

# 事前管理: 统计调用次数，保证不会违规
class VolumeAutoAdjust(LoggerBase):
    def __init__(self, gateway_name: str = ''):
        LoggerBase.__init__(self)
        
        self.gateway_name: str = gateway_name
        self.max_volume: int = math.inf
        self.max_money: Decimal = math.inf
    
    def __call__(self, func):
        def wrapper(self, *args, **kwargs):
            if not self.setting.get('volume_auto_adjust.max_money') is None:
                money = to_decimal(self.setting.get('volume_auto_adjust.max_money'))
                adjust_volume = money / 
                
            ret, data = func(*args, **kwargs)
            return ret, data
        return wrapper
